﻿using ContainerShip.Exceptions;
using ContainerShip.Generics;
using ProjectContainerShip.DrawningObjects;
using ProjectContainerShip.Generics;
using ProjectContainerShip.MovementStrategy;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectContainerShip
{
    /// <summary>
    /// Класс для хранения коллекции
    /// </summary>
    internal class ContainerGenericStorage
    {
        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<ContainerShipCollectionInfo, ContainerGenericCollection<DrawningShip,
        DrawningObjectShip>> _shipStorages;
        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<ContainerShipCollectionInfo> Keys => _shipStorages.Keys.ToList();
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Разделитель для записи ключа и значения элемента словаря
        /// </summary>
        private static readonly char _separatorForKeyValue = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char _separatorRecords = ';';
        /// <summary>
        /// Разделитель для записи информации по объекту в файл
        /// </summary>
        private static readonly char _separatorForObject = ':';
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public ContainerGenericStorage(int pictureWidth, int pictureHeight)
        {
            _shipStorages = new Dictionary<ContainerShipCollectionInfo, ContainerGenericCollection<DrawningShip,
        DrawningObjectShip>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            if (!_shipStorages.ContainsKey(new ContainerShipCollectionInfo(name, string.Empty)))
            {
                var lincornCollection = new ContainerGenericCollection<DrawningShip, DrawningObjectShip>(_pictureWidth, _pictureHeight);
                _shipStorages.Add(new ContainerShipCollectionInfo(name, string.Empty), lincornCollection);
            }
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            if (_shipStorages.ContainsKey(new ContainerShipCollectionInfo(name, string.Empty)))
            {
                _shipStorages.Remove(new ContainerShipCollectionInfo(name, string.Empty));
            }
        }
        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public ContainerGenericCollection<DrawningShip, DrawningObjectShip>? this[string ind]
        {
            get
            {
                ContainerShipCollectionInfo indObj = new ContainerShipCollectionInfo(ind, string.Empty);
                if (_shipStorages.ContainsKey(indObj))
                {
                    return _shipStorages[indObj];
                }
                return null;
            }
        }
        /// <summary>
        /// Сохранение информации в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<ContainerShipCollectionInfo, ContainerGenericCollection<DrawningShip, DrawningObjectShip>> record in _shipStorages)
            {
                StringBuilder records = new();
                foreach (DrawningShip? elem in record.Value.GetShip)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                throw new Exception("Невалидная операция, нет данных для сохранения");
            }
            using (StreamWriter writer = new StreamWriter(filename))
            {
                writer.Write($"shipStorage{Environment.NewLine}{data}");
            }
        }
        /// <summary>
        /// Загрузка информации в хранилище из файла
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new Exception("Файл не найден");
            }
            using (StreamReader reader = new StreamReader(filename))
            {
                string cheker = reader.ReadLine();
                if (cheker == null)
                {
                    throw new Exception("Нет данных для загрузки");
                }
                if (!cheker.StartsWith("shipStorage"))
                {
                    throw new Exception("Неверный формат ввода");
                }
                _shipStorages.Clear();
                string strs;
                bool firstinit = true;
                while ((strs = reader.ReadLine()) != null)
                {
                    if (strs == null && firstinit)
                    {
                        throw new Exception("Нет данных для загрузки");
                    }
                    if (strs == null)
                    {
                        break;
                    }
                    firstinit = false;
                    string name = strs.Split(_separatorForKeyValue)[0];
                    ContainerGenericCollection<DrawningShip, DrawningObjectShip> collection = new(_pictureWidth, _pictureHeight);
                    foreach (string data in strs.Split(_separatorForKeyValue)[1].Split(_separatorRecords))
                    {
                        DrawningShip? ship =
                        data?.CreateDrawningShip(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (ship != null)
                        {
                            try { _ = collection + ship; }
                            catch (ContainerShipNotFoundException e)
                            {
                                throw e;
                            }
                            catch (StorageOverflowException e)
                            {
                                throw e;
                            }
                        }
                    }
                    _shipStorages.Add(new ContainerShipCollectionInfo(name, string.Empty), collection);
                }
            }
        }
    }
}

