﻿using ContainerShip.Exceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectContainerShip.Generics
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetGeneric<T>
    where T : class
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T?> _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Count;
        /// <summary>
        /// Максимальное количество объектов в списке
        /// </summary>
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }
        public void SortSet(IComparer<T?> comparer) => _places.Sort(comparer);
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="ship">Добавляемый контейнеровоз</param>
        /// <returns></returns>
        public bool Insert(T ship, IEqualityComparer<T>? equal = null)
        {
            Insert(ship, 0, equal);
            return true;
        }
        public bool Insert(T ship, int position, IEqualityComparer<T>? equal = null)
        {
            if (position < 0 || position >= _maxCount)
                throw new ContainerShipNotFoundException(position);
            if (Count >= _maxCount)
                throw new StorageOverflowException(position);
            if (equal != null)
            {
                if (_places.Contains(ship, equal))
                    throw new ArgumentException(nameof(ship));
            }
            _places.Insert(position, ship);
            return true;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public bool Remove(int position)
        {
            if (position < 0 || position > _maxCount || position >= Count)
                throw new ContainerShipNotFoundException();
            if (_places[position] == null)
            {
                throw new ContainerShipNotFoundException();
            }
            _places[position] = null;
            return true;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T? this[int position]
        {
            get
            {
                if (position < 0 || position > _maxCount)
                    return null;
                return _places[position];
            }
            set
            {
                if (position < 0 || position > _maxCount)
                    return;
                _places[position] = value;
            }
        }
        /// <summary>
        /// Проход по списку
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T?> GetShip(int? maxShip = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxShip.HasValue && i == maxShip.Value)
                {
                    yield break;
                }
            }
        }
    }
}




